# Copyright (C) 2002-2003, 2008, 2015 The Written Word, Inc.
# Copyright (C) 2000-2001 The Written Word, LLC
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# $Id$

import string
import sys

import pkgsys
import version

from pkgsys import Pkglist
from version import Version

class Depend:
  def __init__ (self):
    self.install = 0			# installation needed

    self.pkg = None			# pkgsys instance

    self.subpkg_name = ''		# subpackage name
    self.test_ver = ''			# dependency version test
    self.test_rev = ''			# dependency revision test
    self.inst_ver = ''			# installed version
    self.inst_rev = ''			# installed revision

def get_dependencies (package, pkgmgr, output, find_installed = 1):
  depends = []				# list of dependencies

  # list of dependencies for package
  pkg_depends = get_dependency_list (package, package.pkgdb_entry_ver)

  # no dependencies
  if not pkg_depends:
    return None

  for pkgname_base in pkg_depends.keys ():
    subpkgs = pkg_depends[pkgname_base].keys ()
    for subpkg in subpkgs:
      # do not add package to installed dependency list if already installed
      dpkg_tests = string.split (pkg_depends[pkgname_base][subpkg])

      pkg = pkgsys.init (package.pkg_type, pkgmgr, output)
      pkg.pkgname_base = pkgname_base
      pkg.subpkgs.append (subpkg)

      depend = Depend ()
      depends.append (depend)

      depend.pkg = pkg

      depend.subpkg_name = subpkg
      depend.test_ver = None
      depend.test_rev = None

      for dpkg_test in dpkg_tests:
        if dpkg_test[:1] == 'v':
          depend.test_ver = dpkg_test
        if dpkg_test[:1] == 'r':
          depend.test_rev = dpkg_test

      if find_installed:
        depend.inst_ver = None
        depend.inst_rev = None

        try:
          iver, irev = pkg.get_installed_pkg_ver \
          (pkg.get_pkgnames (subpkg)[0])
          if not version.met ([depend.test_ver, depend.test_rev],
                              package.make_version (iver, irev),
                              package.make_revision (iver, irev)):
            depend.install = 1

          # record installed version
          depend.inst_ver = iver
          depend.inst_rev = irev
        except pkgsys.PkgsysPackageNotFoundError:
          depend.install = 1
          continue

  return depends

# build dependency list
def get_dependency_list (package, pkgdb_entry_ver):
  pkg_depends = {}

  for subpkg in pkgdb_entry_ver.subpkg.keys ():
    if pkgdb_entry_ver.subpkg[subpkg].has_key ('depend'):
      if subpkg in package.subpkgs:
        depend = pkgdb_entry_ver.subpkg[subpkg]['depend']
        for depend_pkgname in depend.keys ():
          if not pkg_depends.has_key (depend_pkgname):
            pkg_depends[depend_pkgname] = {}

          for depend_subpkg in depend[depend_pkgname].keys ():
            pkg_depends[depend_pkgname][depend_subpkg] = \
            depend[depend_pkgname][depend_subpkg]['test']

  return pkg_depends

# build dependent list
def get_dependent_list (package, pkgdb_entry_ver):
  pkg_dependents = {}

  for subpkg in pkgdb_entry_ver.subpkg.keys ():
    if pkgdb_entry_ver.subpkg[subpkg].has_key ('depend'):
      depend = pkgdb_entry_ver.subpkg[subpkg]['depend']
      if depend.has_key (package.pkgname_base):
        for depend_subpkg in depend[package.pkgname_base].keys ():
          if depend_subpkg in package.subpkgs:
            pkg_dependents[depend_subpkg] = \
            depend[package.pkgname_base][depend_subpkg]['test']

  return pkg_dependents

# get list of packages dependent on package
def get_dependents (all_pkgdbs, package, pkgmgr, subpkgs, output):
  pkgs = Pkglist ()

  for pkgdb_entry in all_pkgdbs:
    for pkgdb_ver in pkgdb_entry.version.keys ():
      pkg_dependents = get_dependent_list (package,
      pkgdb_entry.version[pkgdb_ver])

      # no dependencies
      if not pkg_dependents:
        continue

      for subpkg in pkg_dependents.keys ():
        dpkg_tests = string.split (pkg_dependents[subpkg])
        pkg_dver = pkg_drev = None
        for dpkg_test in dpkg_tests:
          if dpkg_test[:1] == 'v':
            pkg_dver = dpkg_test
          if dpkg_test[:1] == 'r':
            pkg_drev = dpkg_test
      
        if version.met ([pkg_dver, pkg_drev],
                        package.make_version (package.pkg_ver,
                                              package.pkg_rev),
                        package.make_revision (package.pkg_ver,
                                               package.pkg_rev)):
          for pkg in pkgs:
            pkgdb_entry_ver = pkgdb_entry.version[pkgdb_ver]

            if pkg.pkgname_base == pkgdb_entry_ver.pkgname_base:
              break
          else:
            pkg = pkgsys.init (package.pkg_type, pkgmgr, output)
            pkg.pkgdb_entry = pkgdb_entry
            pkg.pkgdb_entry_ver = pkgdb_entry.version[pkgdb_ver]
            pkg.name = pkgdb_entry.prog_name
            pkg.pkgname_base = pkgdb_entry.version[pkgdb_ver].pkgname_base
            pkg.prog_ver = pkgdb_ver.prog_ver

          for pkg_subpkg in pkg.pkgdb_entry_ver.subpkg.keys ():
            if not pkg.pkgdb_entry_ver.subpkg[pkg_subpkg].has_key ('depend'):
              continue

            # add subpackage only if specified as a dependency
            depend = pkg.pkgdb_entry_ver.subpkg[pkg_subpkg]['depend']
            for depend_pkgname in depend.keys ():
              if depend_pkgname == package.pkgname_base and \
              depend[depend_pkgname].has_key (subpkg):
                break
            else:
              continue

            for pkgname in pkg.get_pkgnames (pkg_subpkg):
              try:
                [pkg.pkg_ver, pkg.pkg_rev] = pkg.get_installed_pkg_ver \
                (pkgname)
              except pkgsys.PkgsysPackageNotFoundError:
                continue

              # installed version must match version in pkg-db database.
              # side-effect of this is that if no entry for package
              # exists in pkg-db database, package cannot be removed
              if pkgdb_entry.version[pkgdb_ver].version != pkg.pkg_ver:
                continue
      
              # add package to list as dependency met
              if not pkg in pkgs:
                pkgs.append (pkg)
      
              if pkg_subpkg in subpkgs:
                continue

              if subpkgs:
                if pkg_subpkg not in subpkgs:
                  continue

              if package.pkgname_base == pkg.pkgname_base:
                for _subpkg in pkg.subpkgs:
                  if _subpkg == subpkg:
                    pkg.subpkgs.insert (len (pkg.subpkgs) - 1, pkg_subpkg)
                  break
                else:
                  pkg.subpkgs.append (pkg_subpkg)

              else:
                pkg.subpkgs.append (pkg_subpkg)

  return pkgs
