# Copyright (C) 2001-2003, 2009, 2015 The Written Word, Inc.
# Copyright (C) 2000-2001 The Written Word, LLC
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# $Id$

import os
import sys

maxpos = 72				# max position in line
maxmaxpos = 78				# max position if text can fit
					# on line
class MyIO:
  def __init__ (self, verbose = 0):
    self.last_stream = None
    self.pos = 0			# current position in line
    self.verbose = verbose
    self.first_text = 1
    self.line = ''

  def __pad (self, indent, prefix, line, align, margin):
    # No checks for invalid align values... DTD takes care of that!
    if align == 'center':
      # Pad centered text by half the remaining space to margin
      padding = (margin - indent - len (prefix + line)) / 2
      return padding * ' ' + line
    elif align == 'right':
      # Pad the line out until the right column is at maxmaxpos
      padding = (maxmaxpos - indent - len (prefix + line))
      return padding * ' ' + line
    else:
      return line

  def stdout (self, indent, prefix, msg, verbose_level = 0,
              hangindent = 0, align = 'left'):
    if self.verbose < verbose_level:
      return

    self.write (sys.stdout, indent, prefix, msg, hangindent, align)
    self.last_stream = sys.stdout

  def stderr (self, indent, prefix, msg, verbose_level = 0,
              hangindent = 0, align = 'left'):
    if self.verbose < verbose_level:
      return

    self.write (sys.stderr, indent, prefix, msg, hangindent, align)
    self.last_stream = sys.stderr

  def write (self, stream, indent, prefix, msg, hangindent, align):
    indent_space = ' ' * indent

    # output the prefix on a new line before every other line on stderr
    if self.last_stream == sys.stderr and stream == sys.stderr and \
    self.pos == 0:
      stream.write (prefix + ':\n')

    # append ': ' to prefix if it exists
    if prefix and not hangindent:
      prefix = prefix + ': '

    # prepend old message to existing one until we receive a
    # newline and rescan. if not, then new text that appears
    # after the existing text might appear on the next line
    # even though words might be joined by a comma.
    if msg[-1] == '\n':
      if self.line:
        msg = self.line + msg
        self.line = ''
    else:
      self.line += msg
      return

    # build up each line one word at a time
    while msg:
      space_pos = msg.find (' ')
      if space_pos == -1:
        text = msg
        msg = ''
      else:
        text = msg[:space_pos + 1]
        msg = msg[space_pos + 1:]

      # prefix and indent are always written immediately at the start
      # of a new line, but center aligned text is actually centered in
      # the remaining space following
      if self.pos == 0:
        if hangindent:
          if self.first_text:
            stream.write (indent_space + prefix)
          else:
            stream.write (indent_space + ' ' * len (prefix))

          self.pos = indent + len (prefix)
        else:
          stream.write (indent_space + prefix)
          self.pos = indent + len (prefix)

      self.pos += len (text)

      # if text won't fit on current line, advance to next line
      # and print. However, if text longer than one line, print
      # on current line.
      if self.pos > maxpos and len (text) < (maxpos - indent - len (prefix)):
        if text[-1] == '\n':
          if self.pos <= maxmaxpos:
            stream.write (self.__pad (indent, prefix, self.line + text,
                                      align, maxmaxpos))
            self.line = ''
            self.first_text = 1
            self.pos = 0
            continue

        if not self.first_text:
          stream.write (self.__pad (indent, prefix, self.line,
                        align, maxpos) + '\n')

        if hangindent:
          if self.first_text:
            stream.write (indent_space + prefix)
          else:
            if len (prefix) == 0:
              stream.write ('  ')
            stream.write (indent_space + ' ' * len (prefix))
        else:
          stream.write (indent_space + prefix)
        self.line = text
        self.pos = indent + len (prefix) + len (text)
      else:
        if self.first_text:
          self.first_text = 0
        self.line += text

      # if last bit of text ended in newline, reset position in line
      # to start at beginning. else, more output to come but concatenate
      # to existing output by continuing on the same line.
      if text[-1] == '\n':
        self.first_text = 1
        self.pos = 0

    stream.write (self.__pad (indent, prefix, self.line, align, maxpos))
    self.line = ''
    stream.flush ()
