# Copyright (C) 2001-2004, 2006 The Written Word, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# $Id$

import os
import re
import string
import sys

import pkglinks

def get_common_dirs (install_dir, package, pkgname):
  common_dirs = []
  dirs = []

  if pkgname.endswith ('.sw.base'):
    dirs = ['bin', 'sbin']
  elif pkgname.endswith ('.sw.packages'):
    dirs = ['bin', 'sbin', 'info', 'man']
  elif pkgname.endswith ('.man.pages'):
    dirs = ['man']
  elif pkgname.endswith ('.man.info'):
    dirs = ['info']

  for dir in dirs:
    if os.path.isdir (os.path.join (install_dir, dir)):
      common_dirs.append (dir)
                              
  return common_dirs          

# execute post-install script
def config (package, pkgmgr, systype, pkgnames, output):
  ext = { 'info' : '.info', 'man' : '.man',
          'packages' : '.packages', 'runtime' : '.runtime' }

  # procedure script environment variables
  environ = {}
  for env in ['IFS', 'PATH', 'SHELL', 'TERM', 'TTY', 'TZ']:
    if os.environ.has_key (env):
      environ[env] = os.environ[env]

  # set up environment that make-links script needs
  if pkgmgr.root:
    environ['rbase'] = pkgmgr.root
  else:
    environ['rbase'] = '/'

  for pkgname in pkgnames:
    subpkg_type = package.get_subpkg_type (pkgname)
    if subpkg_type == 'packages':
      install_name = package.pkgdb_entry_ver.install_name + 'p'
    else:
      install_name = package.pkgdb_entry_ver.install_name
    install_path = package.get_install_path (install_name)

    install_dir = os.path.join (pkgmgr.root, package.install_base,
                                install_path)
    tww_inst_dir = os.path.join (install_dir, 'tww-inst')
    common_dirs = get_common_dirs (install_dir, package, pkgname)
    if not common_dirs:
      continue

    # script to create common links. new version has extension on
    # the common 'links-inst' name so removing the runtime subset
    # doesn't remove the script used to generate the links for
    # other subsets
    make_links = os.path.join (tww_inst_dir, 'links-inst' +
                               ext[subpkg_type])
    if not os.path.exists (make_links):
      make_links = os.path.join (tww_inst_dir, 'make-links')

    if os.path.exists (make_links):
      for common_dir in common_dirs:
        apply (output, ['msg', 'installing links in ' +
                        common_dir + ' for ' + pkgname +
                        ' by running:\n'])
        apply (output, ['msg', '$ sh ' + make_links + ' ' +
                        common_dir + '\n', 2])

        # execute make-links script
        os.spawnve (os.P_WAIT, '/bin/sh', ['/bin/sh', make_links,
                                           common_dir], environ)

# execute pre-remove script
def unconfig (package, pkgmgr, systype, pkgnames, output):
  ext = { 'info' : '.info', 'man' : '.man',
          'packages' : '.packages', 'runtime' : '.runtime' }
  links_found = 0

  # procedure script environment variables
  environ = {}
  for env in ['IFS', 'PATH', 'SHELL', 'TERM', 'TTY', 'TZ']:
    if os.environ.has_key (env):
      environ[env] = os.environ[env]

  # set up environment that rm-links script needs
  if pkgmgr.root:
    environ['rbase'] = pkgmgr.root
  else:
    environ['rbase'] = '/'

  apply (output, ['begin', 'searching for common links'])

  for pkgname in pkgnames:
    link_instance = pkglinks.init (package, pkgmgr, systype, pkgname)
    if not link_instance.links:
      continue

    if not links_found:
      apply (output, ['end', 'found'])
      links_found = 1

    subpkg_type = package.get_subpkg_type (pkgname)
    if subpkg_type == 'packages':
      install_name = package.pkgdb_entry_ver.install_name + 'p'
    else:
      install_name = package.pkgdb_entry_ver.install_name
    install_path = package.get_install_path (install_name)

    install_dir = os.path.join (pkgmgr.root, package.install_base,
                                install_path)
    tww_inst_dir = os.path.join (install_dir, 'tww-inst')
    common_dirs = get_common_dirs (install_dir, package, pkgname)
    if not common_dirs:
      continue

    # script to remove common links. new version has extension on
    # the common 'links-inst' name so removing the runtime subset
    # doesn't remove the script used to generate the links for
    # other subsets
    rm_links = os.path.join (tww_inst_dir, 'links-rm' + ext[subpkg_type])
    if not os.path.exists (rm_links):
      rm_links = os.path.join (tww_inst_dir, 'rm-links')

    if os.path.exists (rm_links):
      for common_dir in common_dirs:
        apply (output, ['msg', 'removing links in ' +
                        common_dir + ' for ' + pkgname +
                        ' by running:\n'])
        apply (output, ['msg', '$ sh ' + rm_links + ' ' +
                        common_dir + '\n', 2])
        apply (output, ['msg', '(only info dir updates and errors will \
be displayed)\n', 2])

        # execute rm-links script
        os.spawnve (os.P_WAIT, '/bin/sh', ['/bin/sh', rm_links,
                                           common_dir], environ)

  if not links_found:
    apply (output, ['end', 'none found'])
