# Copyright (C) 2001-2004, 2011 The Written Word, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# $Id$

import os
import re
import string
import sys

# execute postinstall script
def config (package, pkgmgr, systype, pkgnames, output):
  run_procedure_script ('links-inst', 'installing links for',
                        package, pkgnames, output)

# execute preremove script
def unconfig (package, pkgmgr, systype, pkgnames, output):
  run_procedure_script ('links-rm', 'removing links for',
                        package, pkgnames, output)

# execute procedure script
def run_procedure_script (procedure_script_basename,
                          procedure_script_msg, package,
                          pkgnames, output):
  ext = { 'libruntime' : '.librun', 'man' : '.man',
          'packages' : '.pkg', 'runtime' : '.rte' }

  # procedure script environment variables
  environ = {}
  for env in ['IFS', 'PATH', 'SHELL', 'TERM', 'TTY', 'TZ']:
    if os.environ.has_key (env):
      environ[env] = os.environ[env]

  for pkgname in pkgnames:
    subpkg_type = package.get_subpkg_type (pkgname)
    if not subpkg_type in ['libruntime', 'man', 'packages', 'runtime']:
      continue

    if subpkg_type == 'libruntime':
      install_name = package.pkgdb_entry_ver.install_name + 'r'
    elif subpkg_type == 'packages':
      install_name = package.pkgdb_entry_ver.install_name + 'p'
    else:
      install_name = package.pkgdb_entry_ver.install_name
    install_path = package.get_install_path (install_name)

    install_dir = os.path.join (package.install_base, install_path)
    tww_inst_dir = os.path.join (install_dir, 'tww-inst')

    # script to manage common links
    procedure_script = os.path.join (tww_inst_dir,
                                     procedure_script_basename + \
                                     ext[subpkg_type])

    if os.path.exists (procedure_script):
      apply (output, ['msg', procedure_script_msg +
                      pkgname + ' by running:\n'])
      apply (output, ['msg', '$ sh ' + procedure_script + '\n', 2])

      # execute common links script
      os.spawnve (os.P_WAIT, '/bin/sh', ['/bin/sh', procedure_script],
                  environ)
