# Copyright (C) 2001-2004, 2006 The Written Word, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# $Id$

import os
import re
import stat
import string
import sys
import time

import misclib
import pkglinks

from pkgsys import arg_string

# execute post-install script
def config (package, pkgmgr, systype, pkgnames, output):
  args = []
  config_file = ''
  npkg_db = []
  root = pkgmgr.root

  # default root
  if root:
    args.append ('-R')
    args.append (root)

  if pkgmgr.args:
    pkgmgr_args = pkgmgr.args[:]

    while pkgmgr_args:
      arg = pkgmgr_args.pop (0)

      if arg == '-c':
        config_file = pkgmgr_args[0]
      args.append (arg)

  # alternate path to npkg database
  if package.npkg['db']:
    npkg_db.append ('--db-path=' + package.npkg['db'])

  for pkgname in pkgnames:
    # extract postinstall script
    try:
      script, script_path = misclib.mkstempfile ()
    except (IOError, OSError):
      apply (output, ['msg', 'Error while creating temporary file to \
write procedure script to. The temporary file we intended to use \
was "' + script_path + '". The file might exist (possible security \
problem) or there might be other reasons why we could not use it.\n'])
      sys.exit (1)

    for entry in misclib.popen (package.npkg['npkginfo path'],
                                args + npkg_db + ['--systype=' +
                                package.npkg['systype'],
                                '--query=postinstall',
                                pkgname], 'r').readlines ():
      script.write (entry)
    script.close ()

    # if script file empty, no postinstall script exists
    if os.lstat (script_path)[stat.ST_SIZE] == 0:
      continue

    # procedure scripts run in a custom environment. copy from
    # default environment only those variables we want and add
    # new variables to environment required for procedure scripts.
    environ = {}
    for env in ['IFS', 'PATH', 'SHELL', 'TERM', 'TTY', 'TZ']:
      if os.environ.has_key (env):
        environ[env] = os.environ[env]

    # fetch variables to pass to postinstall script
    for pkginfo_db_var in misclib.popen (package.npkg['npkginfo path'],
                                         args + npkg_db + ['--systype=' +
                                         package.npkg['systype'],
                                         '--query=pkginfo-db-vars',
                                         pkgname], 'r').readlines ():
      var, value = string.split (pkginfo_db_var.rstrip (), '=', 2)
      environ[var] = value[1:len (value) - 1]	# strip quotes

    if root:
      environ['NPKG_INSTALL_ROOT'] = root
    if config_file and os.path.exists (config_file):
      environ['NPKG_CONFIG_FILE'] = config_file
    environ['NPKG_BASEDIR'] = root + environ['NPKG_BASEDIR']
    if package.npkg['db']:
      environ['NPKG_DB'] = package.npkg['db']
      environ['NPKG_DBPATH'] = package.npkg['db']
      environ['NPKG_PKGDIR'] = package.npkg['db']

    # Prepend $PATH with path to npkg commands
    if not misclib.find_path_prog ('npkginfo'):
      npkg_bin_dir = os.path.dirname (package.npkg['npkginfo path'])
      environ['PATH'] = npkg_bin_dir + ':' + os.environ['PATH']

    apply (output, ['msg', 'installing links in for ' + pkgname +
                    ' by running:\n'])
    apply (output, ['msg', '$ sh ' + script_path + '\n', 2])

    os.spawnve (os.P_WAIT, '/bin/sh', ['/bin/sh', script_path], environ)
    os.remove (script_path)

# execute pre-remove script
def unconfig (package, pkgmgr, systype, pkgnames, output):
  args = []
  links_found = 0
  npkg_db = []
  root = pkgmgr.root

  # default root
  if root:
    args.append ('-R')
    args.append (root)

  if pkgmgr.args:
    for arg in pkgmgr.args:
      args.append (arg)

  # alternate path to npkg database
  if package.npkg['db']:
    npkg_db.append ('--db-path=' + package.npkg['db'])

  apply (output, ['begin', 'searching for common links'])

  for pkgname in pkgnames:
    link_instance = pkglinks.init (package, pkgmgr, systype, pkgname)
    if not link_instance.links:
      continue

    if not links_found:
      apply (output, ['end', 'found'])
      links_found = 1

    common_dirs = []
    for link in link_instance.links:
      if not link['dir'] in common_dirs:
        common_dirs.append (link['dir'])

    apply (output, ['msg', 'removing links in ' +
                    string.join (common_dirs, ', ') + ' for ' +
                    pkgname + ':\n'])
    for link in link_instance.links:
      apply (output, ['msg', root + link['path'] + '\n', 2])
      os.system (os.path.dirname (package.npkg['npkginfo path']) +
                 '/nremovef ' + arg_string (args) +
                 ' --systype=' + package.npkg['systype'] + ' ' +
                 pkgname + ' ' + link['path'] + ' 2>/dev/null')
      os.remove (root + link['path'])

  if not links_found:
    apply (output, ['end', 'none found'])
