# Copyright (C) 2001-2004, 2006, 2011 The Written Word, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# $Id$

import os
import re
import string
import sys

import pkgconfig
import pkglinks

from basedir import get_common_base

# execute postinstall script
def config (package, pkgmgr, systype, pkgnames, output):
  run_procedure_script ('make-links-', 'installing links for',
                        package, pkgmgr, systype, pkgnames, output)

# execute preremove script
def unconfig (package, pkgmgr, systype, pkgnames, output):
  run_procedure_script ('rm-links-', 'removing links for',
                        package, pkgmgr, systype, pkgnames, output)

# execute procedure script
def run_procedure_script (procedure_script_basename,
                          procedure_script_msg, package,
                          pkgmgr, systype, pkgnames, output):
  for pkgname in pkgnames:
    subpkg_type = package.get_subpkg_type (pkgname)
    if not subpkg_type in ['libruntime', 'man', 'packages', 'runtime']:
      continue

    if package.get_subpkg_type (pkgname) == 'libruntime':
      install_name = package.pkgdb_entry_ver.install_name + 'r'
    elif package.get_subpkg_type (pkgname) == 'packages':
      install_name = package.pkgdb_entry_ver.install_name + 'p'
    else:
      install_name = package.pkgdb_entry_ver.install_name
    install_path = package.get_install_path (install_name)

    install_dir = os.path.join (pkgmgr.root, package.install_base,
                                install_path)
    tww_inst_dir = os.path.join (install_dir, 'tww-inst')
    subpkg_ext = re.sub ('^.*(.)$', '\\1', pkgname)

    # script to manage common links
    procedure_script = os.path.join (tww_inst_dir,
                                     procedure_script_basename + subpkg_ext)

    if os.path.exists (procedure_script):
      # procedure script environment variables
      environ = {}
      for env in ['IFS', 'PATH', 'SHELL', 'TERM', 'TTY', 'TZ']:
        if os.environ.has_key (env):
          environ[env] = os.environ[env]

      # set up environment that make-links script needs
      environ['SUNW_PRODNAME'] = install_name
      environ['PKGINST'] = pkgname
      environ['TWW_COMMON_BASE'] = get_common_base (systype,
                                                    package.install_base,
                                                    pkgname)
      environ['CLIENT_BASEDIR'] = package.install_base
      environ['TWW_INSTALL_PATH'] = install_path
      if pkgmgr.root:
        environ['PKG_INSTALL_ROOT'] = pkgmgr.root
        environ['BASEDIR'] = os.path.join (pkgmgr.root,
                                           package.install_base)
      else:
        environ['BASEDIR'] = package.install_base

      apply (output, ['msg', procedure_script_msg +
                      pkgname + ' by running:\n'])
      apply (output, ['msg', '$ sh ' + procedure_script + '\n', 2])

      # execute make-links script
      os.spawnve (os.P_WAIT, '/bin/sh', ['/bin/sh', procedure_script],
                  environ)
