# Copyright (C) 2001-2004, 2006, 2011 The Written Word, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# $Id$

import os
import re
import string
import sys

import misclib
import pkgconfig

# execute post-install script
def config (package, pkgmgr, systype, pkgnames, output):
  install_base = package.install_base

  # rpm path
  if not pkgmgr.rpm_path:
    rpm_path = misclib.find_path_prog ('rpm')
    if not rpm_path:
      raise pkgconfig.PkgconfigProgNotFound ('rpm', 'rpm-path')
  else:
    rpm_path = pkgmgr.rpm_path

  # rpm database
  args = ''
  if package.rpm['db']:
    args = ' --dbpath ' + package.rpm['db']

  # new installation base if package relocated
  arg_index = 0
  while arg_index < len (pkgmgr.args):
    if pkgmgr.args[arg_index] == '--relocate':
      relocate = pkgmgr.args[arg_index + 1].split ('=', 2)
      if relocate[0] == install_base:
        install_base = relocate[1]
        break

    arg_index += 1

  for pkgname in pkgnames:
    subpkg_type = package.get_subpkg_type (pkgname)
    if not subpkg_type in ['libruntime', 'man', 'packages', 'runtime']:
      continue

    # because RPM packages are relocatable, ignore default
    # install_base and obtain from RPM database
    for entry in os.popen (rpm_path + args + ' -ql ' +
                           pkgname, 'r').readlines ():
      if re.search ('/tww-inst$', entry.rstrip ()):
        install_dir = os.path.dirname (entry.rstrip ())
        break
    else:
      continue

    # extract post-install script
    try:
      script, script_name = misclib.mkstempfile ()
    except (IOError, OSError):
      apply (output, ['msg', 'Error while creating temporary file to \
write RPM procedure script to. The temporary file we intended to use \
was "' + script_name + '". The file might exist (possible security \
problem) or there might be other reasons why we could not use it.\n'])
      sys.exit (1)

    rpm_fd = os.popen (rpm_path + args + ' -q --scripts ' + pkgname, 'r')
    while 1:
      entry = rpm_fd.readline ()
      if not entry:
        break

      # read all lines between 'postinstall script' and 'exit 0'
      if re.match ('^postinstall script', entry):
        while 1:
          entry = rpm_fd.readline ()
          if not entry:
            break

          script.write (entry)
          if re.match ('^exit 0', entry):
            break

    rpm_fd.close ()
    script.close ()

    apply (output, ['msg', 'installing links for ' + pkgname +
                    ' by running:\n'])
    apply (output, ['msg', '$ RPM_INSTALL_PREFIX=' + install_base +
                    ' sh ' + script_name + '\n', 2])

    # post-install script environment variables
    environ = {}
    for env in ['IFS', 'PATH', 'SHELL', 'TERM', 'TTY', 'TZ']:
      if os.environ.has_key (env):
        environ[env] = os.environ[env]

    # set up environment that post-install script needs
    environ['RPM_INSTALL_PREFIX'] = install_base

    # execute post-install script
    os.spawnve (os.P_WAIT, '/bin/sh', ['/bin/sh', script_name], environ)
    os.remove (script_name)

# execute pre-remove script
def unconfig (package, pkgmgr, systype, pkgnames, output):
  install_base = package.install_base

  # rpm path
  if not pkgmgr.rpm_path:
    rpm_path = misclib.find_path_prog ('rpm')
    if not rpm_path:
      raise pkgconfig.PkgconfigProgNotFound ('rpm', 'rpm-path')
  else:
    rpm_path = pkgmgr.rpm_path

  # rpm database
  args = ''
  if package.rpm['db']:
    args = ' --dbpath ' + package.rpm['db']

  # new installation base if package relocated
  arg_index = 0
  while arg_index < len (pkgmgr.args):
    if pkgmgr.args[arg_index] == '--relocate':
      relocate = pkgmgr.args[arg_index + 1].split ('=', 2)
      if relocate[0] == install_base:
        install_base = relocate[1]
        break

    arg_index += 1

  for pkgname in pkgnames:
    subpkg_type = package.get_subpkg_type (pkgname)
    if not subpkg_type in ['libruntime', 'man', 'packages', 'runtime']:
      continue

    # because RPM packages are relocatable, ignore default
    # install_base and obtain from RPM database
    for entry in os.popen (rpm_path + args + ' -ql ' +
                           pkgname, 'r').readlines ():
      if re.search ('/tww-inst$', entry.rstrip ()):
        install_dir = os.path.dirname (entry.rstrip ())
        break
    else:
      continue

    # extract pre-remove script
    try:
      script, script_name = misclib.mkstempfile ()
    except (IOError, OSError):
      apply (output, ['msg', 'Error while creating temporary file to \
write RPM procedure script to. The temporary file we intended to use \
was "' + script_name + '". The file might exist (possible security \
problem) or there might be other reasons why we could not use it.\n'])
      sys.exit (1)

    rpm_fd = os.popen (rpm_path + args + ' -q --scripts ' + pkgname, 'r')
    while 1:
      entry = rpm_fd.readline ()
      if not entry:
        break

      # read all lines between 'preuninstall script' and 'exit 0'
      if re.match ('^preuninstall script', entry):
        while 1:
          entry = rpm_fd.readline ()
          if not entry:
            break

          script.write (entry)
          if re.match ('^exit 0', entry):
            break

    rpm_fd.close ()
    script.close ()

    apply (output, ['msg', 'removing links for ' + pkgname +
                    ' by running:\n'])
    apply (output, ['msg', '$ RPM_INSTALL_PREFIX=' + install_base +
                    ' sh ' + script_name + '\n', 2])

    # pre-remove script environment variables
    environ = {}
    for env in ['IFS', 'PATH', 'SHELL', 'TERM', 'TTY', 'TZ']:
      if os.environ.has_key (env):
        environ[env] = os.environ[env]

    # set up environment that pre-remove script needs
    environ['RPM_INSTALL_PREFIX'] = install_base

    # execute pre-remove script
    os.spawnve (os.P_WAIT, '/bin/sh', ['/bin/sh', script_name,
                                       '0'], environ)
    os.remove (script_name)
