# Copyright (C) 2001-2003, 2005-2006, 2010, 2015 The Written Word, Inc.
# Copyright (C) 2000-2001 The Written Word, LLC
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# $Id$

import os
import sys

import pkginst

from misclib import md5sum

# constants
prefix = '/opt/TWWfsw/pkgutils16'
contribdir = prefix + '/lib/aux'

# if verbose description of PkginstArchiveError exception printed
_invalid_archive_error_msg_printed = False

class Archive:
  def __init__ (self, path, pkgdir = None):
    self.path = path				# path to archive
    self.pkgdir = pkgdir			# path to temp directory
    self.payload_file = 'data.gcpio.bz2'	# payload filaname
    self.bzip2_path = os.path.join (contribdir, 'bin', 'bzip2')
    self.gcpio_path = os.path.join (contribdir, 'bin', 'gcpio')

  def verify_md5_checksum (self, member, local_depot_is_temporary,
                           local_depot):
    # member is relative to pwd
    pwd = os.getcwd ()
    os.chdir (self.pkgdir)

    try:
      if not self.exists (member):
        os.chdir (pwd)
        return 0
    except pkginst.PkginstArchiveError, msg:
      os.chdir (pwd)
      raise

    # if checksum file does not exist, error
    try:
      if not self.exists (member + '.md5'):
        os.chdir (pwd)
        raise pkginst.PkginstMD5ChecksumNotFoundError ('MD5 checksum \
file ' + member + '.md5 does not exist in archive\n')
    except pkginst.PkginstArchiveError, msg:
      os.chdir (pwd)
      raise

    if not self.extract (member, local_depot_is_temporary,
                         local_depot):
      os.chdir (pwd)
      return 0
    if not self.extract (member + '.md5', local_depot_is_temporary,
                         local_depot):
      os.chdir (pwd)
      return 0

    # calculate
    member_cksum = md5sum (member)
    fh = open (member + '.md5', 'r')
    cksum = fh.readline ().rstrip ()
    fh.close ()
    if member_cksum == cksum:
      os.chdir (pwd)
      return 1

    # remove file as checksum didn't match
    os.remove (member)
    os.remove (member + '.md5')

    os.chdir (pwd)
    return 0

def init (path, pkgdir = None):
  global _invalid_archive_error_msg_printed

  # determine archive type
  fh = open (path, 'rb')
  archive_type = fh.read (6)
  fh.close ()

  # ar(1) archive
  if archive_type == '!<arch':
    exec 'import pkginst.archive.ar'
    exec 'archive = pkginst.archive.ar.ArchiveAr (path, pkgdir)'
  # cpio(1) archive
  elif archive_type == '070701':
    exec 'import pkginst.archive.gcpio'
    exec 'archive = pkginst.archive.gcpio.ArchiveGcpio (path, pkgdir)'
  # plain xml archive
  elif archive_type == '<?xml ':
    exec 'import pkginst.archive.xmlplain'
    exec 'archive = pkginst.archive.xmlplain.ArchiveXML (path, pkgdir)'
  # zip archive
  elif archive_type == '\x50\x4b\x03\x04\x14\x00':
    exec 'import pkginst.archive.zip'
    exec 'archive = pkginst.archive.zip.ArchiveZip (path, pkgdir)'
  else:
    if _invalid_archive_error_msg_printed:
      raise pkginst.PkginstArchiveError ('unable to determine file \
type of archive package "' + path + '"')
    else:
      _invalid_archive_error_msg_printed = True
      raise pkginst.PkginstArchiveError ('unable to determine file \
type of archive package "' + path + '". when a list of package \
names is specified on the command-line, a check is performed to \
determine whether or not they are archive packages. this is done \
by looking for a file of the same name on the local file system. \
if the file is found relative to the current path, it is considered \
an archive package.')

  return archive
