# Copyright (C) 2001-2003, 2006, 2010 The Written Word, Inc.
# Copyright (C) 2000-2001 The Written Word, LLC
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# $Id$

import os
import sys

import misclib
import pkginst

from pkginst.archive import Archive

class ArchiveGcpio (Archive):
  def exists (self, member):
    if not os.path.exists (self.gcpio_path):
      raise pkginst.PkginstArchiveProgNotFoundError ('GNU cpio program ' +
      self.gcpio_path + ' not found.\n')

    output = os.popen (self.gcpio_path + ' -it --file ' + self.path + \
    ' ' + member + ' 2>&1')
    while 1:
      line = output.readline ().rstrip ()
      if not line:
        break

      if line == member:
        output.close ()
        return 1

    output.close ()
    return 0

  def extract (self, member):
    # extract to package directory
    pwd = os.getcwd ()
    os.chdir (self.pkgdir)

    if not os.path.exists (member):
      if os.system (self.gcpio_path + ' -iu --file ' + self.path + ' ' + \
      member + ' 2>/dev/null'):
        os.chdir (pwd)
        return 0

    os.chdir (pwd)
    return 1

  def unpack_payload (self, local_depot_is_temporary, local_depot):
    # extract to package directory
    pwd = os.getcwd ()
    os.chdir (self.pkgdir)

    bzip2 = os.popen (self.bzip2_path + ' -q -dc ' +
    self.payload_file + ' 2>/dev/null', 'r', 0)
    cpio = os.popen (self.gcpio_path + ' -i 2>/dev/null', 'w')
    while 1:
      chunk = bzip2.read (misclib.READ_CHUNK)
      if not chunk:
        break
      cpio.write (chunk)
    cpio_ret = cpio.close ()
    bzip2_ret = bzip2.close ()

    os.chdir (pwd)
    if cpio_ret or bzip2_ret:
      raise pkginst.PkginstExtractError ('error using bzip2 | gcpio \
to unpack ' + payload_file)

  def verify_payload (self):
    if os.system (self.bzip2_path + ' --test ' + self.payload_file +
    ' >/dev/null 2>&1'):
      raise pkginst.PkginstExtractError ('error using bzip2 to unpack ' +
      self.payload_file)
