# Copyright (C) 2001-2004, 2006 The Written Word, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# $Id$

import os
import re
import string
import sys

import misclib
import pkglinks

from pkglinks import Pkglinks

class npkg (Pkglinks):
  def __init__ (self, package, pkgmgr, systype, pkgname):
    Pkglinks.__init__ (self, package, pkgmgr, systype, pkgname)

    args = []
    npkg_db = []
    root = pkgmgr.root

    self.npkg = package.npkg

    # alternate path to npkg database
    if self.npkg['db']:
      npkg_db.append ('--db-path=' + self.npkg['db'])

    # default root
    if root:
      args.append ('-R')
      args.append (root)

    if pkgmgr.args:
      for arg in pkgmgr.args:
        args.append (arg)

    # if COMMON_BASE=none in config file
    if self.common_base == 'none':
      return

    # fetch variables to pass to postinstall script
    for pkginfo_db_var in misclib.popen (self.npkg['npkginfo path'],
                                         args + npkg_db + ['--systype=' +
                                         self.npkg['systype'],
                                         '--query=pkginfo-db-vars',
                                         pkgname], 'r').readlines ():
      var, value = string.split (pkginfo_db_var.rstrip (), '=', 2)
      if var == 'TWW_COMMON_DEP_BASE':
        self.common_base = value[1:len (value) - 1]	# strip quotes
        break

    # locate tww-inst directory
    listing = misclib.popen (self.npkg['npkginfo path'],
                             args + npkg_db + ['--systype=' +
                             self.npkg['systype'], '--list-files',
                             '--print=pathname', '--print=type',
                             '--print=link-source', '--print=checksum',
                             pkgname], 'r').readlines ()
    while listing:
      entry = listing.pop (0).rstrip ()
      if not re.match ('Pathname:', entry):
        continue

      link = {}
      link['path'] = re.sub ('^[^\s]+ (.*)', '\\1', entry)

      # what common directory link is for
      if re.match ('Pathname: ' + self.common_base + 'bin/', entry) or \
      re.match ('Pathname: ' + self.common_base + 'sbin/', entry):
        link['dir'] = 'bin'
      else:
        continue

      while listing:
        # empty link marks end of file entry
        entry = listing.pop (0).rstrip ()
        if entry == '':
          break

        if re.match ('Type: symbolic link', entry):
          link['type'] = 'l'

          while listing:
            entry = listing.pop (0).rstrip ()

            if re.match ('Destination of link:', entry):
              link['dest'] = re.sub ('^[^:]+: (.*)', '\\1', entry)
              self.links.append (link)
              break
        elif re.match ('Type: regular file', entry):
          link['type'] = 'w'

          while listing:
            entry = listing.pop (0).rstrip ()

            if re.match ('.*checksum:', entry):
              link['checksum'] = re.sub ('^[^:]+: (.*)', '\\1', entry)
              self.links.append (link)
              break
