# Copyright (C) 2001-2003, 2006 The Written Word, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# $Id$

import os
import re
import sys

import misclib
import pkglinks

from pkglinks import Pkglinks

class rpm (Pkglinks):
  def __init__ (self, package, pkgmgr, systype, pkgname):
    Pkglinks.__init__ (self, package, pkgmgr, systype, pkgname)

    # if COMMON_BASE=none in config file
    if self.common_base == 'none':
      return

    # make sure rpm is in $PATH
    if not pkgmgr.rpm_path:
      rpm_path = misclib.find_path_prog ('rpm')
      if not rpm_path:
        raise pkglinks.PkglinksProgNotFound ('rpm', 'rpm-path')
    else:
      rpm_path = pkgmgr.rpm_path

    rpm_db = ''
    if pkgmgr.rpm_db:
      rpm_db = '--dbpath ' + pkgmgr.rpm_db + ' '

    # locate tww-inst directory
    for entry in os.popen (rpm_path + ' ' + rpm_db +
    '-ql ' + self.pkgname, 'r').readlines ():
      if entry.rstrip ().endswith ('/tww-inst'):
        links_file = entry.rstrip () + '/links'
        break
    else:
      return None

    try:
      fh = open (links_file, 'r')
    except IOError:
      return None
    while 1:
      entry = fh.readline ().rstrip ()
      if not entry: break

      link = {}
      link['dir'] = re.sub ('^. ([^ ]+) .*', '\\1', entry)

      if self.pkgname.endswith ('-pkg'):
        if link['dir'] not in [ 'bin', 'info', 'man', 'sbin' ]:
          continue
      elif self.pkgname.endswith ('-man'):
        if link['dir'] not in [ 'info', 'man' ]:
          continue
      elif self.pkgname.endswith ('-conf') or \
      self.pkgname.endswith ('-doc') or \
      self.pkgname.endswith ('-librun'):
        continue
      else:
        if link['dir'] not in [ 'bin', 'sbin' ]:
          continue

      link['type'] = re.sub ('^(.) .*', '\\1', entry)
      link['path'] = re.sub ('^. [^ ]+ ([^ ]+) .*', '\\1', entry)
      if link['type'] == 'l':
        link['dest'] = re.sub ('^. [^ ]+ [^ ]+ ([^ ]+) .*', '\\1', entry)
        link['local file'] = re.sub ('^. [^ ]+ [^ ]+ [^ ]+ ([^ ]+)',
                                     '\\1', entry)
      elif link['type'] == 'w' or link['type'] == 'W':
        link['checksum'] = re.sub ('^. [^ ]+ [^ ]+ ([^ ]+) .*', '\\1', entry)

      self.links.append (link)
    fh.close ()
