# Copyright (C) 2002-2003 The Written Word, Inc.
# Copyright (C) 2000-2001 The Written Word, LLC
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# $Id$

import string
import re

# version number and revision class
class Version:
  def __init__ (self, ver):
    if not ver:
      self.ver = ['0']
    else:
      self.ver = str (ver).split ('.')

  def __nonzero__ (self):
    for ver in self.ver:
      if ver != '0':
	return 1
    else:
      return 0

  def __cmp__ (self, other):
    ver1 = []
    ver2 = []
    for x in self.ver:
      ver1.append (x)
    for x in other.ver:
      ver2.append (x)

    cmp = None
    while ver1:
      if ver2:
	ver1_num = ver1[0]
	ver2_num = ver2[0]
	del ver1[0]
	del ver2[0]

	# we compare first components of version. set to -1 if
	# less than, 0 if equal to, and 1 if greater than. as
	# more components are tested, if the result is the same,
	# continue, else stop because we have our result.

	# convert string to integer for numeric comparisons
	if re.match ('^\d+$', ver1_num) and re.match ('^\d+$', ver2_num):
	  if int (ver1_num) < int (ver2_num):
	    new_cmp = -1
	  elif int (ver1_num) == int (ver2_num):
	    new_cmp = 0
	  else:
	    new_cmp = 1
	else:
	  if str (ver1_num) < str (ver2_num):
	    new_cmp = -1
	  elif str (ver1_num) == str (ver2_num):
	    new_cmp = 0
	  else:
	    new_cmp = 1

	if not cmp:
	  if new_cmp == 0:
	    cmp = new_cmp
	  else:
	    return (new_cmp)
	else:
	  if new_cmp != 0: return (new_cmp)

      else:
        return (1)

    if not ver1 and ver2:
      return -1
    return 0

  def __str__ (self):
    return (string.join (self.ver, '.'))
  def __add__ (self, arg):
    return (string.join (self.ver, '.') + arg)
  def __radd__ (self, arg):
    return (arg + string.join (self.ver, '.'))

# return most recent version and release of a package in a depot
def latest (dep, pkg_name, pkgname_base = None):
  pkg_ver = None		# Version number specific to package
  pkg_rev = None		# management system
  pkg_subpkg = None		# package subpackages
  latest_prog_ver = None	# Program version number

  pkgdb_entry = dep.get_pkgdb_entry (pkg_name)
  for pkgdb_ver in pkgdb_entry.version.keys ():
    # if matching against package basename as well
    if pkgname_base and \
    pkgdb_entry.version[pkgdb_ver].pkgname_base != pkgname_base:
      continue

    prog_ver = pkgdb_ver.prog_ver

    # get version and release of package (version might be
    # different depending on package management system)
    version = pkgdb_entry.version[pkgdb_ver].version
    revision = pkgdb_entry.version[pkgdb_ver].revision
    subpkg = pkgdb_entry.version[pkgdb_ver].subpkg

    if not pkg_ver:
      pkg_ver = version
      pkg_rev = revision
      pkg_subpkg = subpkg
      latest_prog_ver = prog_ver
    elif version > pkg_ver or (version == pkg_ver and revision > pkg_rev):
      pkg_ver = version
      pkg_rev = revision
      pkg_subpkg = subpkg
      latest_prog_ver = prog_ver

  return [latest_prog_ver, pkg_ver, pkg_rev, pkg_subpkg]

# test if program version or package release version meets
# dependency requirement
def met (ver_tests, pkg_ver, pkg_rev):
  for ver_test in ver_tests:
    if not ver_test:
      continue

    var = re.sub ('^(\w)([>=<]+)(.*)', '\\1', ver_test)
    test = re.sub ('^(\w)([>=<]+)(.*)', '\\2', ver_test)
    ver = Version (re.sub ('^(\w)([>=<]+)(.*)', '\\3', ver_test))

    if var == 'v':
      test_ver = pkg_ver
    if var == 'r':
      test_ver = pkg_rev

    if test == '>' and test_ver <= ver:
      return None
    if test == '>=':
      if test_ver < ver:
        return None
      if test_ver > ver:
        return 1
    if test == '==' and test_ver != ver:
      return None
    if test == '<=' and test_ver > ver:
      return None
    if test == '<' and test_ver >= ver:
      return None

  return 1
